/+ 
 ' Functions that gather important dates during Easter. 
 ' 1) getAshWednesday()
 ' 2) getPalmSunday()
 ' 3) getMaundyThursday()
 ' 4) getGoodFriday()
 ' 5) getEasterSunday()
 ' 6) getAscensionThursday()
 ' 7) getPentecost()
 '
 ' License: Public Domain
 '
 ' To Compile: C:\dmd\MKoD_ex>dmd getEasterDates.d
 '
 ' 02.Oct.04 Compiled and Tested with dmd v0.102
 ' 04.Jun.06 Compiled and Tested with dmd v0.160
 ' 10.Jan.07 Compiled and Tested with dmd v1.0
 '
 +/
private import std.stdio;

/+
 ' Determine the date of "Easter Sunday" for any year  
 ' in the Gregorian Calendar from 1583 to 4099. 
 '
 +/
void getEasterSunday
(
    in  int iyear, 
    out int imonth, 
    out int iday
)
{
    iday = getEasterDays( iyear );
       
    if ( iday > 31 )
    {
        iday   = iday - 31;
        imonth = 4;
    }
    else 
        imonth = 3;
        
} // end void getEasterSunday( in int, out int, out int )

/+
 ' Determine the date of "Palm Sunday" for any year  
 ' in the Gregorian Calendar from 1583 to 4099. 
 '
 +/
void getPalmSunday
(
    in  int iyear, 
    out int imonth, 
    out int iday
)
{
    iday = getEasterDays( iyear ) - 7;
    
    if ( iday > 31 )
    {
        iday   = iday - 31;
        imonth = 4;
    }
    else 
        imonth = 3;
    
} // end void getPalmSunday( in int, out int, out int )

/+
 ' Determine the date of "Good Friday" for any year  
 ' in the Gregorian Calendar from 1583 to 4099. 
 '
 +/
void getGoodFriday
(
    in  int iyear, 
    out int imonth, 
    out int iday
)
{
    iday = getEasterDays( iyear ) - 2;    
    
    if ( iday > 31 )
    {
        iday   = iday - 31;
        imonth = 4;
    }
    else 
        imonth = 3;    
        
} // end void getGoodFriday( in int, out int, out int )

/+
 ' Determine the date of "Maundy Thursday" for any year  
 ' in the Gregorian Calendar from 1583 to 4099. 
 '
 +/
void getMaundyThursday
(
    in  int iyear, 
    out int imonth, 
    out int iday
)
{
    iday = getEasterDays( iyear ) - 3;    
    
    if ( iday > 31 )
    {
        iday   = iday - 31;
        imonth = 4;
    }
    else 
        imonth = 3;    
        
} // end void getMaundyThursday( in int, out int, out int )

/+
 ' Determine the date of "Ash Wednesday" for any year  
 ' in the Gregorian Calendar from 1583 to 4099. 
 '
 +/
void getAshWednesday
(
    in  int iyear, 
    out int imonth, 
    out int iday
)
{
    iday = getEasterDays( iyear ) - 46;    
     
    if ( iday > 0 )
    {
        imonth = 3;
    }
    else
    { 
        iday   = 28 + iday + leapYearDay( iyear );
        imonth = 2;    
    }
        
} // end void getAshWednesday( in int, out int, out int )

/+
 ' Determine the date of "Ascension Thursday" for any year  
 ' in the Gregorian Calendar from 1583 to 4099. 
 '
 +/
void getAscensionThursday
(
    in  int iyear, 
    out int imonth, 
    out int iday
)
{
    iday = getEasterDays( iyear ) + 40;
       
    if ( iday > 93 )
    {
        iday   = iday - 93;
        imonth = 6;
    }
    else if (iday > 62 )
    {
        iday = iday - 62;
        imonth = 5;
    }    
    else 
        imonth = 4;
          
} // end void getAscensionThursday( in int, out int, out int )

/+
 ' Determine the date of "Pentecost" for any year  
 ' in the Gregorian Calendar from 1583 to 4099. 
 '
 +/
void getPentecost
(
    in  int iyear, 
    out int imonth, 
    out int iday
)
{
    iday = getEasterDays( iyear ) + 50;
       
    if ( iday > 93 )
    {
        iday   = iday - 93;
        imonth = 6;
    }
    else if (iday > 62 )
    {
        iday = iday - 62;
        imonth = 5;
    }    
    else if ( iday > 31 )
    {
        iday = iday - 31;
        imonth = 4;
    }    
    else
        imonth = 3;        
    
} // end void getPentecost( in int, out int, out int )

private int getEasterDays
(
    in  int iyear 
)
{
    int ia; 
    int ib; 
    int ic;
    int itA;
    int itB; 
    int itC; 
    int itD; 
    int itE;
    int iday = 0;

    ia = iyear / 100;
    ib = iyear % 19;

    ic = ( ( ia - 15 ) >> 1 ) + 202 - 11 * ib;
    
    if ( ia > 26 ) ic--;
    if ( ia > 38 ) ic--;
    if ( ia == 21 || ia == 24 || ia == 25 || 
         ia == 33 || ia == 36 || ia==37 ) 
        ic--;
        
    ic  = ic % 30;
    itA = ic + 21;
    
    if ( ic == 29 ) itA--;
    if ( ic == 28 && ib > 10) itA--;
    
    itB = ( itA - 19 ) % 7;
    ic  = ( 40 - ia ) & 3;
    itC = ic;
    
    if ( ic > 1) itC++;
    if ( ic == 3) itC++;
    
    ic  = iyear % 100;
    itD = ( ic + ( ic >> 2 ) ) % 7;
    itE = ( ( 20 - itB - itC - itD ) % 7 ) + 1;
    iday = itA + itE;
    
    return iday;
    
} // end int getEasterDays( in int )

private int leapYearDay
(
    in int iyear
)
{
    return ( ( iyear & 3 ) == 0 && ( iyear % 100 || ( iyear % 400 ) == 0 ) );
    
} // end int leapYearDay( in int )

int main()
{
    char[][ 6 ] sMonths;
    sMonths[][ 0 ] = "January";
    sMonths[][ 1 ] = "February";
    sMonths[][ 2 ] = "March"; 
    sMonths[][ 3 ] = "April"; 
    sMonths[][ 4 ] = "May";
    sMonths[][ 5 ] = "June";
    
    int iAshWedMonth       = 0;
    int iAshWedDay         = 0;
    int iPalmSunMonth      = 0;
    int iPalmSunDay        = 0;
    int iMaundyThrMonth    = 0;
    int iMaundyThrDay      = 0;
    int iGoodFriMonth      = 0;
    int iGoodFriDay        = 0;
    int iEasterSunMonth    = 0;
    int iEasterSunDay      = 0;
    int iAscensionThrMonth = 0;
    int iAscensionThrDay   = 0;
    int iPentecostMonth    = 0;
    int iPentecostDay      = 0;
    
    for ( int iyear = 1996; iyear <= 2008; iyear++ )
    { 
        getAshWednesday( iyear, iAshWedMonth, iAshWedDay );
        getPalmSunday( iyear, iPalmSunMonth, iPalmSunDay );
        getMaundyThursday(  iyear, iMaundyThrMonth, iMaundyThrDay );
        getGoodFriday( iyear, iGoodFriMonth, iGoodFriDay );
        getEasterSunday( iyear, iEasterSunMonth, iEasterSunDay );
        getAscensionThursday( iyear, iAscensionThrMonth, iAscensionThrDay );
        getPentecost( iyear, iPentecostMonth, iPentecostDay );
        
        writefln( "For %d AshWednesday       is on %s %d\n"
                  "         Palm Sunday        is on %s %d\n"
                  "         Maundy Thursday    is on %s %d\n"
                  "         Good Friday        is on %s %d\n"
                  "         Easter Sunday      is on %s %d\n"
                  "         Ascension Thursday is on %s %d\n"
                  "         Pentecost          is on %s %d", 
                  iyear,
                  sMonths[ iAshWedMonth - 1  ], iAshWedDay, 
                  sMonths[ iPalmSunMonth - 1 ], iPalmSunDay, 
                  sMonths[ iMaundyThrMonth - 1  ], iMaundyThrDay, 
                  sMonths[ iGoodFriMonth - 1 ], iGoodFriDay, 
                  sMonths[ iEasterSunMonth - 1 ], iEasterSunDay,
                  sMonths[ iAscensionThrMonth - 1 ], iAscensionThrDay,
                  sMonths[ iPentecostMonth - 1 ], iPentecostDay ); 
              
        writefln();
    }    
 
    return 0;
    
} // end int main()
