/+
 ' Title           : Fun with Prime Numbers  
 ' Original Author : Steve Litt (c) 2004 All rights reserved. 
 ' Original Code   : http://www.troubleshooters.com/codecorn/primenumbers/primenumbers.htm
 ' Example         : Getting Prime Numbers by using the "Array Method (Sieve of Eratosthenes)"
 ' -------------------------------------------------------------------------------------------
 ' D conversion by David 'SpottedTiger' Davis - last tested with D v0.160
 ' Sample Primes: 2, 3, 5, 7, 11, 13, 17, 19, 23, 29, ...
 +/
module findprime;

private import std.stdio;

void printPrime( in uint uiPrime )
{
    static uint uiy = 0U;

    std.stdio.writef( "%4d, ", uiPrime );
    uiy++;

    if ( ( uiy % 10U ) == 0U )
    {
        uiy = 0U;
        std.stdio.writefln();  
    }
} // end void printPrime( in ulong )

void findPrimes( in uint uiTopCandidate )
{
    char[] sPrimeArray;
    uint   uiThisFactor;
    uint   uiLastSquare;
    uint   uiThisSquare;
    uint   uiMark;

    sPrimeArray.length =  uiTopCandidate + 1U;
    sPrimeArray[] = true;
      
    // Set all but 0 and 1 to Prime status    
    sPrimeArray[ 0 ] = 0U;
    sPrimeArray[ 1 ] = 0U;
    
    // Mark all the non-Primes
    uiThisFactor = 2U;
    uiLastSquare = 0U;
    uiThisSquare = 0U;
    
    while ( uiThisFactor * uiThisFactor <= uiTopCandidate )
    { 
        // Mark the multiples of this factor
        uiMark = uiThisFactor + uiThisFactor;
        
        while( uiMark <= uiTopCandidate )
        {
            sPrimeArray[ uiMark ] = false;
            uiMark += uiThisFactor;
        } 
         
        // Print the proven Primes so far 
        uiThisSquare = uiThisFactor * uiThisFactor;
        
        for ( ; uiLastSquare < uiThisSquare; uiLastSquare++ )
            if( sPrimeArray[ uiLastSquare ] == true ) 
                printPrime( uiLastSquare );
                
        // Set ulThisFactor to the Prime
        uiThisFactor++;
        
        while ( sPrimeArray[ uiThisFactor ] == false ) 
            uiThisFactor++;
            
        assert( uiThisFactor <= uiTopCandidate );
    } 
    
    // Print the remaining Primes
    for ( ; uiLastSquare <= uiTopCandidate; uiLastSquare++ )
        if( sPrimeArray[ uiLastSquare ] == true ) 
            printPrime( uiLastSquare );
    
    sPrimeArray.length = 0U;
    
} // end void findPrimes( in uint )

int main()
{
    uint uiTopCandidate = 1000U;
    
    writefln( "All the numbers listed below between 0 to 1000 are Primes!" );
    findPrimes( uiTopCandidate );
    
    return 0;
    
} // end int main()